@@ -1,4 +1,5 @@
 require 'tempfile'
+require_relative './env'
 
 module Thin
   # Raised when an incoming request is not valid
@@ -13,16 +14,16 @@ module Thin
     BODY_TMPFILE      = 'thin-body'.freeze
     MAX_HEADER        = 1024 * (80 + 32)
 
-    INITIAL_BODY      = ''
+    INITIAL_BODY      = String.new
     # Force external_encoding of request's body to ASCII_8BIT
-    INITIAL_BODY.encode!(Encoding::ASCII_8BIT) if INITIAL_BODY.respond_to?(:encode!)
+    INITIAL_BODY.encode!(Encoding::ASCII_8BIT) if INITIAL_BODY.respond_to?(:encode!) && defined?(Encoding::ASCII_8BIT)
 
     # Freeze some HTTP header names & values
     SERVER_SOFTWARE   = 'SERVER_SOFTWARE'.freeze
     SERVER_NAME       = 'SERVER_NAME'.freeze
     REQUEST_METHOD    = 'REQUEST_METHOD'.freeze
     LOCALHOST         = 'localhost'.freeze
-    HTTP_VERSION      = 'HTTP_VERSION'.freeze
+    REQUEST_HTTP_VERSION      = 'thin.request_http_version'.freeze
     HTTP_1_0          = 'HTTP/1.0'.freeze
     REMOTE_ADDR       = 'REMOTE_ADDR'.freeze
     CONTENT_LENGTH    = 'CONTENT_LENGTH'.freeze
@@ -52,30 +53,26 @@ module Thin
 
     def initialize
       @parser   = Thin::HttpParser.new
-      @data     = ''
+      @data     = String.new
       @nparsed  = 0
       @body     = StringIO.new(INITIAL_BODY.dup)
-      @env      = {
+      @env      = Env.with_defaults({
         SERVER_SOFTWARE   => SERVER,
         SERVER_NAME       => LOCALHOST,
 
         # Rack stuff
         RACK_INPUT        => @body,
-
-        RACK_VERSION      => VERSION::RACK,
         RACK_ERRORS       => STDERR,
-
-        RACK_MULTITHREAD  => false,
-        RACK_MULTIPROCESS => false,
-        RACK_RUN_ONCE     => false
-      }
+      })
     end
 
     # Parse a chunk of data into the request environment
     # Raises an +InvalidRequest+ if invalid.
     # Returns +true+ if the parsing is complete.
     def parse(data)
-      if @parser.finished?  # Header finished, can only be some more body
+      if data.size > 0 && finished? # headers and body already fully satisfied. more data is erroneous.
+        raise InvalidRequest, 'Content longer than specified'
+      elsif @parser.finished?  # Header finished, can only be some more body
         @body << data
       else                  # Parse more header using the super parser
         @data << data
@@ -112,7 +109,7 @@ module Thin
       # Clients and servers SHOULD NOT assume that a persistent connection
       # is maintained for HTTP versions less than 1.1 unless it is explicitly
       # signaled. (http://www.w3.org/Protocols/rfc2616/rfc2616-sec8.html)
-      if @env[HTTP_VERSION] == HTTP_1_0
+      if @env[REQUEST_HTTP_VERSION] == HTTP_1_0
         @env[CONNECTION] =~ KEEP_ALIVE_REGEXP
 
       # HTTP/1.1 client intends to maintain a persistent connection unless
